@@ -13,22 +13,23 @@ function parse (args, opts) {
   // aliases might have transitive relationships, normalize this.
   const aliases = combineAliases(Object.assign(Object.create(null), opts.alias))
   const configuration = Object.assign({
-    'short-option-groups': true,
-    'camel-case-expansion': true,
-    'dot-notation': true,
-    'parse-numbers': true,
     'boolean-negation': true,
-    'negation-prefix': 'no-',
+    'camel-case-expansion': true,
+    'combine-arrays': false,
+    'dot-notation': true,
     'duplicate-arguments-array': true,
     'flatten-duplicate-arrays': true,
-    'populate--': false,
-    'combine-arrays': false,
-    'set-placeholder-key': false,
+    'greedy-arrays': true,
     'halt-at-non-option': false,
+    'nargs-eats-options': false,
+    'negation-prefix': 'no-',
+    'parse-numbers': true,
+    'populate--': false,
+    'set-placeholder-key': false,
+    'short-option-groups': true,
     'strip-aliased': false,
     'strip-dashed': false,
-    'unknown-options-as-args': false,
-    'nargs-eats-options': false
+    'unknown-options-as-args': false
   }, opts.configuration)
   const defaults = Object.assign(Object.create(null), opts.default)
   const configObjects = opts.configObjects || []
@@ -303,6 +304,12 @@ function parse (args, opts) {
           }
         }
       }
+    } else if (arg.match(/^-[0-9]$/) &&
+      arg.match(negative) &&
+      checkAllAliases(arg.slice(1), flags.bools)) {
+      // single-digit boolean alias, e.g: xargs -0
+      key = arg.slice(1)
+      setArg(key, defaultValue(key))
     } else if (arg === '--') {
       notFlags = args.slice(i + 1)
       break
@@ -346,7 +353,6 @@ function parse (args, opts) {
   }
 
   if (configuration['strip-aliased']) {
-    // XXX Switch to [].concat(...Object.values(aliases)) once node.js 6 is dropped
     ;[].concat(...Object.keys(aliases).map(k => aliases[k])).forEach(alias => {
       if (configuration['camel-case-expansion']) {
         delete argv[alias.split('.').map(prop => camelCase(prop)).join('.')]
@@ -360,7 +366,10 @@ function parse (args, opts) {
   // on the nargs option?
   function eatNargs (i, key, args) {
     let ii
-    const toEat = checkAllAliases(key, flags.nargs)
+    let toEat = checkAllAliases(key, flags.nargs)
+    // NaN has a special meaning for the array type, indicating that one or
+    // more values are expected.
+    toEat = isNaN(toEat) ? 1 : toEat
 
     if (toEat === 0) {
       setArg(key, defaultValue(key))
@@ -396,6 +405,8 @@ function parse (args, opts) {
   function eatArray (i, key, args) {
     let argsToSet = []
     let next = args[i + 1]
+    // If both array and nargs are configured, enforce the nargs count:
+    const nargsCount = checkAllAliases(key, flags.nargs)
 
     if (checkAllAliases(key, flags.bools) && !(/^(true|false)$/.test(next))) {
       argsToSet.push(true)
@@ -412,13 +423,16 @@ function parse (args, opts) {
         if (/^-/.test(next) && !negative.test(next) && !isUnknownOptionAsArg(next)) break
         i = ii
         argsToSet.push(processValue(key, next))
+        if (!configuration['greedy-arrays'] ||
+            (nargsCount && argsToSet.length >= nargsCount)) break
       }
     }
 
     // If both array and nargs are configured, create an error if less than
-    // nargs positionals were found:
-    const toEat = checkAllAliases(key, flags.nargs)
-    if (toEat && argsToSet.length < toEat) {
+    // nargs positionals were found. NaN has special meaning, indicating
+    // that at least one value is required (more are okay).
+    if ((nargsCount && argsToSet.length < nargsCount) ||
+        (isNaN(nargsCount) && argsToSet.length === 0)) {
       error = Error(__('Not enough arguments following: %s', key))
     }
 
@@ -465,6 +479,7 @@ function parse (args, opts) {
       const keys = [key].concat(flags.aliases[key] || [])
       keys.forEach(function (key) {
         Object.defineProperty(argvReturn, key, {
+          enumerable: true,
           get () {
             return val
           },
@@ -682,6 +697,10 @@ function parse (args, opts) {
     if (!configuration['dot-notation']) keys = [keys.join('.')]
 
     keys.slice(0, -1).forEach(function (key, index) {
+      // TODO(bcoe): in the next major version of yargs, switch to
+      // Object.create(null) for dot notation:
+      key = sanitizeKey(key)
+
       if (typeof o === 'object' && o[key] === undefined) {
         o[key] = {}
       }
@@ -701,7 +720,10 @@ function parse (args, opts) {
       }
     })
 
-    const key = keys[keys.length - 1]
+    // TODO(bcoe): in the next major version of yargs, switch to
+    // Object.create(null) for dot notation:
+    const key = sanitizeKey(keys[keys.length - 1])
+
     const isTypeArray = checkAllAliases(keys.join('.'), flags.arrays)
     const isValueArray = Array.isArray(value)
     let duplicate = configuration['duplicate-arguments-array']
@@ -986,4 +1008,11 @@ Parser.detailed = function (args, opts) {
   return parse(args.slice(), opts)
 }
 
+// TODO(bcoe): in the next major version of yargs, switch to
+// Object.create(null) for dot notation:
+function sanitizeKey (key) {
+  if (key === '__proto__') return '___proto___'
+  return key
+}
+
 module.exports = Parser
